package config

import (
	"errors"
	logger "eta/eta_mini_ht_api/common/component/log"
	mappingDao "eta/eta_mini_ht_api/models/config"
	"gorm.io/gorm"
)

type CustomerProductRiskMappingDTO struct {
	CustomerRiskLevel string
	ProductRiskLevel  string
}

func GetRiskMappingByCustomerRiskLevel(customerRiskLevel string) (dto CustomerProductRiskMappingDTO, err error) {
	if customerRiskLevel == "" {
		err = errors.New("客户风险等级不能为空")
		return
	}
	mapping, err := mappingDao.GetCustomerRiskMapping(customerRiskLevel)
	if err != nil {
		if errors.Is(err, gorm.ErrRecordNotFound) {
			logger.Error("产品风险等级未配置")
		} else {
			logger.Error("查询产品风险等级配置失败")
		}
		return
	}
	dto = convertToDTO(mapping)
	return
}
func convertToDTO(mapping mappingDao.CustomerProductRiskMapping) CustomerProductRiskMappingDTO {
	return CustomerProductRiskMappingDTO{
		CustomerRiskLevel: mapping.CustomerRisk,
		ProductRiskLevel:  mapping.ProductRisk,
	}
}
