package report

import (
	"encoding/json"
	"eta/eta_mini_ht_api/common/component/cache"
	logger "eta/eta_mini_ht_api/common/component/log"
	"eta/eta_mini_ht_api/common/contants"
	"eta/eta_mini_ht_api/common/utils/redis"
	"eta/eta_mini_ht_api/domian/report"
	"eta/eta_mini_ht_api/models/eta"
	"eta/eta_mini_ht_api/models/ht"
	"eta/eta_mini_ht_api/task/base"
	"sync"
	"time"
)

var (
	updateTaskName base.TaskType = "ReportUpdateSyncTask"
	updateCron                   = "0/10 * * * * *"
	redisCache     *cache.RedisCache
)

func rd() *cache.RedisCache {
	if redisCache == nil {
		redisCache = cache.GetInstance()
	}
	return redisCache
}

// Execute Task ETA取研报的数据
func (re *ReportUpdateTask) Execute(taskDetail *base.TaskDetail) error {
	logger.Info(contants.TaskFormat, "更新研报库开始")
	var wg sync.WaitGroup
	wg.Add(2)
	go func() {
		defer wg.Done()
		var etaReportList []eta.ETAReport
		etaReportList, err := eta.GetUpdateETAReports()
		if err != nil {
			logger.Error("获取ETA研报列表失败：%v", err)
		}
		if len(etaReportList) > 0 {
			var list []byte
			list, err = json.Marshal(etaReportList)
			if err == nil {
				taskDetail.Content = string(list)
			}
			err = report.SyncETAReportList(etaReportList)
			if err != nil {
				logger.Error("同步ETA研报列表失败：%v", err)
			}
		}
		logger.Info(contants.TaskFormat, "同步ETA研报库结束")
	}()
	go func() {
		defer wg.Done()
		var htReportList []ht.HTReport
		var filterReportList []ht.HTReport
		htReportList, err := ht.GetUpdateHTReports()
		for i := 0; i < len(htReportList); i++ {
			key := redis.GenerateReportRefreshKey("HT", htReportList[i].Id, int64(htReportList[i].PublishTime))
			if rd().SetIfNotExist(key, "processed", 3600) {
				logger.Info("同步HT报告 ID :%d,修改时间:%v", htReportList[i].Id, int64(htReportList[i].PublishTime))
				filterReportList = append(filterReportList, htReportList[i])
			} else {
				logger.Info("过滤HT重复的消息 ID :%d,修改时间:%v", htReportList[i].Id, int64(htReportList[i].PublishTime))
			}
		}
		if err != nil {
			logger.Error("获取HT研报列表失败：%v", err)
		}
		if len(filterReportList) > 0 {
			for i := 0; i < len(filterReportList); i++ {
				timestamp := int64(filterReportList[i].PublishTime)
				t := time.UnixMilli(timestamp)
				filterReportList[i].PublishedTime = t.Format(time.DateTime)
				plateId := filterReportList[i].PlateId
				var plate ht.HTPlate
				plate, err = ht.GetPermissionNameById(plateId)
				if err != nil || plate.ParentId == 0 {
					filterReportList[i].PermissionName = filterReportList[i].PlateName
				} else {
					var PermissionName string
					err = getPermissionNameById(plate.Id, &PermissionName)
					if err != nil {
						logger.Error("获取HT研报列表失败：%v", err)
						filterReportList[i].PermissionName = ""
					} else {
						filterReportList[i].PermissionName = PermissionName
					}
				}
			}
			var stop bool
			stop, err = report.SyncHTReportList(filterReportList)
			if err != nil {
				logger.Error("同步HT研报列表失败：%v", err)
			}
			if stop {
				logger.Info(contants.TaskFormat, "同步HT研报库结束")
			}
		} else {
			logger.Info(contants.TaskFormat, "同步HT研报库结束")
		}
	}()
	wg.Wait()
	return nil
}

func getPermissionNameById(id int, currentName *string) (err error) {
	plate, err := ht.GetPermissionNameById(id)
	if err != nil {
		logger.Error("查询海通板块品种名称失败：%v", err)
		return
	}
	if plate.ParentId != 0 {
		*currentName = plate.PlateName
		return getPermissionNameById(plate.ParentId, currentName)
	} else {
		return
	}
}

type ReportUpdateTask struct {
}

func init() {
	reportTask := base.NewTask(updateTaskName, updateCron, new(ReportUpdateTask), base.PROD)
	base.RegisterTask(&reportTask)
}
